'use strict';

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _opn = require('opn');

var _opn2 = _interopRequireDefault(_opn);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

var _index = require('./index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var argv = _yargs2.default.usage('Usage: $0 [...options]').options({
  init: {
    type: 'string',
    description: 'Initialize a sample project'
  },
  port: {
    default: 3030,
    description: 'Port to listen on'
  },
  run: {
    alias: 'r',
    type: 'string',
    description: 'Script to run in browser'
  },
  watch: {
    alias: 'w',
    type: 'array',
    description: 'Scripts to watch for changes'
  },
  bundle: {
    alias: 'b',
    type: 'array',
    description: 'Scripts to bundle'
  },
  html: {
    type: 'string',
    description: 'Name of the input file for sharable HTML bundle'
  },
  js: {
    type: 'string',
    description: 'Name of the JavaScript file to include in the HTML bundle when no HTML is specified'
  },
  output: {
    alias: 'o',
    type: 'string',
    description: 'Name of the output file'
  },
  common: {
    type: 'string',
    description: 'Name of file to contain common code in case of multiple entries'
  },
  production: {
    type: 'boolean',
    default: false,
    description: 'Optimize bundle for production'
  },
  sourcemaps: {
    type: 'boolean',
    default: true,
    description: 'Generate sourcemaps for bundle'
  },
  browser: {
    type: 'string',
    default: '',
    description: 'Name of using browser'
  }
}).example('$0 --run index.js', 'Run the script \'index.js\' in a browser and watch for changes').example('$0 --port 8008 --watch index.js', 'Start the server in the port \'8008\' and watch \'index.js\' for changes').example('$0 --bundle entry.js --output bundle.js --production', 'Generate a bundle named \'bundle.js\' from \'entry.js\' for production').example('$0 --html index.html --output bundle.html', 'Generate a sharable HTML file named \'bundle.html\' from \'index.html\'').help('help').version(_package2.default.version).strict().argv;

if (argv.init) {
  (0, _index.init)({
    root: process.cwd(),
    name: argv.init
  }).then(function () {
    console.log('\u2B50  All done!\n\n' + (' - Run ' + _chalk2.default.bold(_chalk2.default.yellow('npm start')) + ' to start the server\n') + (' - Run ' + _chalk2.default.bold(_chalk2.default.yellow('npm run build')) + ' to build production ready bundle\n'));
  }).catch(function (err) {
    console.log(err.message);
    process.exit(1);
  });
} else if (argv.bundle) {
  (0, _index.bundle)({
    root: process.cwd(),
    entry: argv.bundle.map(function (it) {
      return './' + it;
    }),
    common: argv.common,
    output: argv.output,
    production: argv.production,
    sourcemaps: argv.sourcemaps
  }).then(function (result) {
    var assets = result.assetsByChunkName;
    var bundles = [];

    for (var b in assets) {
      var entry = assets[b];

      bundles.push(_path2.default.resolve(process.cwd(), Array.isArray(entry) ? entry[0] : entry));
    }

    console.log('\u2B50  Bundle' + (bundles.length > 1 ? 's' : '') + ' generated at ' + bundles.map(function (b) {
      return _chalk2.default.bold(_chalk2.default.green(b));
    }).join(', '));
  }).catch(function (err) {
    console.error(err);
    process.exit(1);
  });
} else if (typeof argv.html === 'string') {
  (0, _index.html)({
    root: process.cwd(),
    js: argv.js,
    entry: argv.html,
    output: argv.output,
    production: argv.production,
    sourcemaps: argv.sourcemaps
  }).then(function (file) {
    console.log('\u2B50  Sharable HTML generated at ' + _chalk2.default.bold(_chalk2.default.green(file)));
  }).catch(function (err) {
    console.error(err);
    process.exit(1);
  });
} else {
  (0, _index.server)({
    root: process.cwd(),
    port: argv.port,
    run: argv.run,
    watch: argv.watch
  }).listen(argv.port);

  var url = 'http://localhost:' + argv.port;

  console.log('Quik is serving files at ' + _chalk2.default.blue(url));

  if (argv.browser) {
    (0, _opn2.default)(url, { app: argv.browser });
  } else {
    (0, _opn2.default)(url);
  }
}
//# sourceMappingURL=quik-cli.js.map